#include <unistd.h>

#include "Process.h"
#include "Socket.h"
#include "Exception.h"


//----------------------------------------------------------------------------
class Server : public Process
{
public:
    Server();
    ~Server();

    int run();

protected:
    void onTerm();

    TCPUnixListener m_listener;
    bool m_shutdown;
};


//----------------------------------------------------------------------------
Server::Server()
{
    static const char *socket = "/tmp/glt05_socket";

    unlink(socket);
    m_listener.bind(UnixAddress(socket));
    m_listener.listen();

    m_shutdown = false;
}

//----------------------------------------------------------------------------
Server::~Server()
{
}

//----------------------------------------------------------------------------
void Server::onTerm()
{
    std::cout << "Caught a SIGTERM." << std::endl;
    m_shutdown = true;
}

//----------------------------------------------------------------------------
int Server::run()
{
    UnixAddress addr;
    char buffer[1024];

    std::cout << "Server listening on "
              << m_listener.getLocalAddress(addr) << std::endl;

    while (!m_shutdown)
    {
        std::auto_ptr<TCPUnixConnection> c(m_listener.accept());
        if (c->getFD() == -1 && errno == EINTR)
        {
            std::cout << "accept() returned -1, errno == EINTR" << std::endl;
            continue;
        }

        int rc = c->recv(buffer, sizeof(buffer));

        std::cout << "Received message." << std::endl;
        c->send(buffer, rc);
    }

    std::cout << "Server going down..." << std::endl;

    return 0;
}



//----------------------------------------------------------------------------
int main()
{
    try
    {
        Server s;
        return s.run();
    }
    catch (Exception &e)
    {
        std::cerr << "Server error: " << e << std::endl;
    }
}
