#include <unistd.h>

#include "Process.h"
#include "SystemCalls.h"
#include "Socket.h"
#include "Exception.h"


//----------------------------------------------------------------------------
class Server : public Process
{
public:
    Server();
    ~Server();

    int run();

protected:
    void onTerm();

    UDPListener m_listener;
    bool m_shutdown;
};


//----------------------------------------------------------------------------
Server::Server()
{
    m_listener.bind(InetAddress("0.0.0.0", 10000));
    m_shutdown = false;
}

//----------------------------------------------------------------------------
Server::~Server()
{
}

//----------------------------------------------------------------------------
void Server::onTerm()
{
    std::cout << "Caught a SIGTERM." << std::endl;
    m_shutdown = true;
}

//----------------------------------------------------------------------------
int Server::run()
{
    InetAddress addr;
    char buffer[1024];
    bool parent = true;
    static const unsigned NUM_CHILDS = 4;

    for (unsigned i=0; i<NUM_CHILDS; i++)
    {
        if (SystemCall::fork() == 0)
        {
            parent = false;
            break;
        }
    }

    if (!parent)
    {
        std::cout << getpid() << ": Server listening on "
                  << m_listener.getLocalAddress(addr) << std::endl;

        while (!m_shutdown)
        {
            int rc = m_listener.recvfrom(buffer, sizeof(buffer), &addr);
            if (rc == -1 && errno == EINTR)
            {
                continue;
            }

            std::cout << getpid() << ": Received message from "
                      << addr << std::endl;
            m_listener.sendto(buffer, rc, addr);
        }
    }
    else
    {
        std::cout << "Parent waiting for shutdown." << std::endl;
        for (unsigned i=0; i<NUM_CHILDS; i++)
        {
            int rc = SystemCall::wait(NULL);
            std::cout << "wait() returns: " << rc << std::endl;
            if (rc == -1 && errno == EINTR)
            {
                continue;
            }
        }

        std::cout << "Server going down..." << std::endl;
    }

    return 0;
}



//----------------------------------------------------------------------------
int main()
{
    try
    {
        Server s;
        return s.run();
    }
    catch (Exception &e)
    {
        std::cerr << "Server error: " << e << std::endl;
    }
}
