#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <sstream>

#include "SystemCalls.h"
#include "Address.h"


//----------------------------------------------------------------------------
Address::Address()
{
}

//----------------------------------------------------------------------------
Address::~Address()
{
}

//----------------------------------------------------------------------------
std::ostream &operator<<(std::ostream &os, const Address &addr)
{
    return addr.writeTo(os);
}




//----------------------------------------------------------------------------
InetAddress::InetAddress()
{
    memset(&m_addr, 0, sizeof(m_addr));
    m_addr.sin_family = AF_INET;
}

//----------------------------------------------------------------------------
InetAddress::InetAddress(const char *addr, uint16_t port)
{
    memset(&m_addr, 0, sizeof(m_addr));
    m_addr.sin_family = AF_INET;
    setAddress(addr);
    setPort(port);
}

//----------------------------------------------------------------------------
InetAddress::~InetAddress()
{
}


//----------------------------------------------------------------------------
void InetAddress::setAddress(const char *addr)
{
    inet_aton(addr, &m_addr.sin_addr);
}

//----------------------------------------------------------------------------
const char *InetAddress::getAddress() const
{
    return inet_ntoa(m_addr.sin_addr);
}

//----------------------------------------------------------------------------
void InetAddress::setPort(uint16_t port)
{
    m_addr.sin_port = htons(port);
}

//----------------------------------------------------------------------------
uint16_t InetAddress::getPort() const
{
    return ntohs(m_addr.sin_port);
}


//----------------------------------------------------------------------------
struct sockaddr *InetAddress::getSockAddr()
{
    return reinterpret_cast<struct sockaddr*>(&m_addr);
}

//----------------------------------------------------------------------------
const struct sockaddr *InetAddress::getSockAddr() const
{
    return reinterpret_cast<const struct sockaddr*>(&m_addr);
}

//----------------------------------------------------------------------------
socklen_t InetAddress::getSockLen() const
{
    return sizeof(struct sockaddr_in);
}


//----------------------------------------------------------------------------
std::ostream &InetAddress::writeTo(std::ostream &os) const
{
    return os << getAddress() << ":" << getPort();
}

//----------------------------------------------------------------------------
std::string InetAddress::toString() const
{
    std::ostringstream os;
    writeTo(os);
    return os.str();
}



//----------------------------------------------------------------------------
HostAddress::HostAddress() : InetAddress()
{
}

//----------------------------------------------------------------------------
HostAddress::HostAddress(const char *addr, uint16_t port)
        : InetAddress(addr, port)
{
}

//----------------------------------------------------------------------------
HostAddress::HostAddress(const char *host)
{
    setHost(host);
}

//----------------------------------------------------------------------------
HostAddress::~HostAddress()
{
}

//----------------------------------------------------------------------------
void HostAddress::setHost(const char *addr)
{
    struct hostent *host = SystemCall::gethostbyname(addr);
    memcpy(&m_addr.sin_addr, host->h_addr, sizeof(m_addr.sin_addr));
}



//----------------------------------------------------------------------------
UnixAddress::UnixAddress() : Address()
{
    memset(&m_addr, 0, sizeof(m_addr));
    m_addr.sun_family = AF_UNIX;
}

//----------------------------------------------------------------------------
UnixAddress::UnixAddress(const char *path) : Address()
{
    memset(&m_addr, 0, sizeof(m_addr));
    m_addr.sun_family = AF_UNIX;
    setPath(path);
}

//----------------------------------------------------------------------------
UnixAddress::~UnixAddress()
{
}


//----------------------------------------------------------------------------
void UnixAddress::setPath(const char *path)
{
    strncpy(m_addr.sun_path, path, sizeof(m_addr.sun_path));
}

//----------------------------------------------------------------------------
const char *UnixAddress::getPath() const
{
    return m_addr.sun_path;
}


//----------------------------------------------------------------------------
struct sockaddr *UnixAddress::getSockAddr()
{
    return reinterpret_cast<struct sockaddr*>(&m_addr);
}

//----------------------------------------------------------------------------
const struct sockaddr *UnixAddress::getSockAddr() const
{
    return reinterpret_cast<const struct sockaddr*>(&m_addr);
}

//----------------------------------------------------------------------------
socklen_t UnixAddress::getSockLen() const
{
    return sizeof(struct sockaddr_un);
}


//----------------------------------------------------------------------------
std::ostream &UnixAddress::writeTo(std::ostream &os) const
{
    return os << m_addr.sun_path;
}

//----------------------------------------------------------------------------
std::string UnixAddress::toString() const
{
    std::ostringstream os;
    writeTo(os);
    return os.str();
}
