#ifndef ADDRESS_H
#define ADDRESS_H

#include <stdint.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/un.h>

#include <iostream>

//----------------------------------------------------------------------------
class Address
{
  public:
    //------------------------------------------------------------------------
    Address();
    virtual ~Address();

    //------------------------------------------------------------------------
    virtual struct sockaddr *getSockAddr() = 0;
    virtual const struct sockaddr *getSockAddr() const = 0;
    virtual socklen_t getSockLen() const = 0;

    //------------------------------------------------------------------------
    virtual std::ostream &writeTo(std::ostream &os) const = 0;
    virtual std::string toString() const = 0;

    //------------------------------------------------------------------------
    friend std::ostream &operator<<(std::ostream &os, const Address &addr);
};


//----------------------------------------------------------------------------
class InetAddress : public Address
{
  public:
    //------------------------------------------------------------------------
    InetAddress();
    InetAddress(const char *addr, const uint16_t port);
    virtual ~InetAddress();

    //------------------------------------------------------------------------
    void setAddress(const char *addr);
    const char *getAddress() const;

    void setPort(const uint16_t port);
    uint16_t getPort() const;

    //------------------------------------------------------------------------
    struct sockaddr *getSockAddr();
    const struct sockaddr *getSockAddr() const;
    socklen_t getSockLen() const;

    //------------------------------------------------------------------------
    std::ostream &writeTo(std::ostream &os) const;
    std::string toString() const;

  protected:
    //------------------------------------------------------------------------
    struct sockaddr_in m_addr;
};


//----------------------------------------------------------------------------
class HostAddress : public InetAddress
{
  public:
    //------------------------------------------------------------------------
    HostAddress();
    HostAddress(const char *addr, const uint16_t port);
    HostAddress(const char *host);
    virtual ~HostAddress();

    //------------------------------------------------------------------------
    void setHost(const char *addr);
};


//----------------------------------------------------------------------------
class UnixAddress : public Address
{
  public:
    //------------------------------------------------------------------------
    UnixAddress();
    UnixAddress(const char *path);
    ~UnixAddress();

    //------------------------------------------------------------------------
    void setPath(const char *path);
    const char *getPath() const;

    //------------------------------------------------------------------------
    struct sockaddr *getSockAddr();
    const struct sockaddr *getSockAddr() const;
    socklen_t getSockLen() const;

    //------------------------------------------------------------------------
    std::ostream &writeTo(std::ostream &os) const;
    std::string toString() const;

  protected:
    //------------------------------------------------------------------------
    struct sockaddr_un m_addr;
};

#endif //ADDRESS_H
