#include <string.h>
#include <netdb.h>

#include <sstream>

#include "Exception.h"


//----------------------------------------------------------------------------
SystemCallException::SystemCallException(const char *syscall)
        : Exception()
        , m_syscall(syscall)
        , m_errno(errno)
{
}

//----------------------------------------------------------------------------
SystemCallException::~SystemCallException()
{
    m_syscall = NULL;
}

//----------------------------------------------------------------------------
std::string SystemCallException::toString() const
{
    std::ostringstream os;
    os << m_syscall << ": " << strerror(m_errno);
    return os.str();
}


//----------------------------------------------------------------------------
bool SystemCallException::isError(int rc)
{
    if (rc == -1)
    {
        switch (errno)
        {
          case EINTR:
            break;

          default:
            return true;
        }
    }

    return false;
}



//----------------------------------------------------------------------------
SocketException::SocketException(const char *syscall)
        : SystemCallException(syscall)
{
}

//----------------------------------------------------------------------------
SocketException::~SocketException()
{
}

//----------------------------------------------------------------------------
bool SocketException::isError(int rc)
{
    if (rc == -1)
    {
        switch (errno)
        {
          case EINTR:
          case EAGAIN:
          case EINPROGRESS:
            break;

          default:
            return true;
        }
    }

    return false;
}



//----------------------------------------------------------------------------
GetHostByNameException::GetHostByNameException(const char *name)
        : Exception()
        , m_name(name)
        , m_herrno(h_errno)
{
}

//----------------------------------------------------------------------------
GetHostByNameException::~GetHostByNameException()
{
}

//----------------------------------------------------------------------------
std::string GetHostByNameException::toString() const
{
    std::ostringstream os;
    os << m_name << ": " << hstrerror(m_herrno);
    return os.str();
}
