#ifndef EXCEPTION_H
#define EXCEPTION_H

#include <errno.h>
#include <iostream>

//----------------------------------------------------------------------------
class Exception
{
  public:
    Exception() {}
    Exception(const std::string &text) : m_text(text) {}
    virtual ~Exception() {}

    inline void setText(const std::string &text) { m_text = text; }

    virtual std::string toString() const { return m_text; }

    friend std::ostream &operator<<(std::ostream &os, const Exception &e)
    {
        return os << e.toString();
    }

  private:
    std::string m_text;
};


//----------------------------------------------------------------------------
class SystemCallException : public Exception
{
  public:
    SystemCallException(const char *syscall);
    virtual ~SystemCallException();

    virtual std::string toString() const;

    static bool isError(int rc);

  protected:
    const char *m_syscall;
    int m_errno;
};


//----------------------------------------------------------------------------
class SocketException : public SystemCallException
{
  public:
    SocketException(const char *syscall);
    virtual ~SocketException();

    static bool isError(int rc);

  protected:
    const char *m_syscall;
    int m_errno;
};


//----------------------------------------------------------------------------
class GetHostByNameException : public Exception
{
  public:
    GetHostByNameException(const char *name);
    virtual ~GetHostByNameException();

    virtual std::string toString() const;

  protected:
    const char *m_name;
    int m_herrno;
};

#endif //SOCKETEXCEPTION_H
