#include <cassert>

#include "Poller.h"


//----------------------------------------------------------------------------
PollerCallBackInterface::PollerCallBackInterface()
{
}

//----------------------------------------------------------------------------
PollerCallBackInterface::~PollerCallBackInterface()
{
}




//----------------------------------------------------------------------------
Poller::Poller(PollerCallBackInterface *caller)
        : m_caller(caller)
{
    m_pollFDs.reserve(64);
    m_selectedPollFDs.reserve(64);
}

//----------------------------------------------------------------------------
Poller::~Poller()
{
    m_caller = NULL;
}

//----------------------------------------------------------------------------
void Poller::add(int fd, pollevent_t events)
{
    IndexMap::const_iterator iter = m_index.find(fd);
    if (iter == m_index.end())
    {
        // Create a new entry.

        struct pollfd p;
        p.fd = fd;
        p.events = events;
        p.revents = 0;

        m_index[fd] = m_pollFDs.size();
        m_pollFDs.push_back(p);
    }
    else
    {
        // Overwrite the existing entry.

        m_pollFDs[iter->second].events = events;
    }
}

//----------------------------------------------------------------------------
void Poller::addEvents(int fd, pollevent_t events)
{
    IndexMap::const_iterator iter = m_index.find(fd);
    if (iter != m_index.end())
    {
        m_pollFDs[iter->second].events |= events;
    }
    else
    {
        assert(false);
    }
}

//----------------------------------------------------------------------------
void Poller::delEvents(int fd, pollevent_t events)
{
    IndexMap::const_iterator iter = m_index.find(fd);
    if (iter != m_index.end())
    {
        m_pollFDs[iter->second].events &= ~events;
    }
    else
    {
        assert(false);
    }
}

//----------------------------------------------------------------------------
void Poller::remove(int fd)
{
    IndexMap::iterator iter = m_index.find(fd);
    if (iter != m_index.end())
    {
        if (iter->second < m_pollFDs.size()-1)
        {
            // Overwrite the entry to remove with the last entry.
            m_pollFDs[iter->second] = m_pollFDs.back();
            m_index[m_pollFDs.back().fd] = iter->second;
        }

        // Delete the last entry.
        m_pollFDs.pop_back();
        m_index.erase(iter);
    }
    else
    {
        assert(false);
    }
}

//----------------------------------------------------------------------------
int Poller::poll(int timeout)
{
    int rc = ::poll(&m_pollFDs.front(), m_pollFDs.size(), timeout);
    int rc2 = rc;

    // Copy all selected pollfd entries to m_selectedPollFDs.
    m_selectedPollFDs.clear();
    for (size_t i=0; rc2>0 && i<m_pollFDs.size(); i++)
    {
        if (m_pollFDs[i].revents)
        {
            m_selectedPollFDs.push_back(m_pollFDs[i]);
            rc2--;
        }
    }

    // Callback for each selected pollfd.
    for (size_t i=0; i<m_selectedPollFDs.size(); i++)
    {
        m_caller->onPoll(m_selectedPollFDs[i].fd,
                         m_selectedPollFDs[i].revents);
    }

    return rc;
}
