#ifndef POLLER_H
#define POLLER_H

#include <sys/poll.h>

#include <vector>
#include <map>

//----------------------------------------------------------------------------
typedef short int pollevent_t;

//----------------------------------------------------------------------------
class PollerCallBackInterface
{
    //------------------------------------------------------------------------
    friend class Poller;

  public:
    //------------------------------------------------------------------------
    PollerCallBackInterface();
    virtual ~PollerCallBackInterface();

  protected:
    //------------------------------------------------------------------------
    virtual void onPoll(int fd, pollevent_t events) = 0;
};

//----------------------------------------------------------------------------
class Poller
{
    //------------------------------------------------------------------------
    typedef std::vector<struct pollfd> PollFDVector;
    typedef std::map<int, size_t> IndexMap;

  public:
    //------------------------------------------------------------------------
    Poller(PollerCallBackInterface *caller);
    ~Poller();

    //------------------------------------------------------------------------
    void add(int fd, pollevent_t events);
    void addEvents(int fd, pollevent_t events);
    void delEvents(int fd, pollevent_t events);
    void remove(int fd);

    //------------------------------------------------------------------------
    int poll(int timeout = -1);

  protected:
    //------------------------------------------------------------------------
    PollerCallBackInterface *m_caller;

    //------------------------------------------------------------------------
    /// A vector of pollfd structs, used for the poll() call.
    PollFDVector m_pollFDs;

    /// A helper vector, which will be filled with all selected sockets.
    PollFDVector m_selectedPollFDs;

    /// m_index[fd] is the index of fd's pollfd struct in the m_pollFDs vector.
    IndexMap m_index;
};

#endif //POLLER_H
