#ifndef SYSTEMCALLS_H
#define SYSTEMCALLS_H

#include <sys/types.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netdb.h>

#include "Address.h"

//----------------------------------------------------------------------------
namespace SystemCall
{
    int accept(int s, Address *addr);

    void bind(int s, const Address &addr);

    void close(int s);

    int connect(int s, const Address &addr);

    int fcntl(int fd, int cmd);

    int fcntl(int fd, int cmd, long arg);

    pid_t fork();

    struct hostent *gethostbyname(const char *name);

    void getpeername(int s, Address &addr);

    void getsockname(int s, Address &addr);

    void getsockopt(int s, int level, int optname,
                    void *optval, socklen_t *optlen);

    void listen(int s, int backlog);

    int recv(int s, void *buf, size_t len, int flags);

    int recvfrom(int s, void *buf, size_t len, int flags, Address *addr);

    int recvmsg(int s, struct msghdr *msg, int flags);

    int send(int s, const void *buf, size_t len, int flags);

    int sendto(int s, const void *buf, size_t len, int flags, const Address &addr);

    int sendmsg(int s, const struct msghdr *msg, int flags);

    void setsockopt(int s, int level, int optname,
                    const void *optval, socklen_t optlen);

    int socket(int domain, int type, int protocol);

    void socketpair(int d, int type, int protocol, int *sv);

    pid_t wait(int *status);
}

#endif //SYSTEMCALLS_H
